function [ sysr, V, W, info ] = lpvprojmr(sys, nr, varargin)
%        [ RSYS, V, W, INFO ] = LPVPROJMR(SYS, ORDERS, METHOD)
%
%   RSYS = PROJRED(SYS,ORDERS) computes a reduced-order approximation RSYS
%   of the LPV model SYS. The desired order (number of states) for RSYS
%   is specified by ORDERS. You can try multiple orders at once by setting
%   ORDERS to a vector of integers, in which case RSYS is an array of
%   reduced-order models. When SYS is unstable poles, the normalized coprime
%   factor reduction method can be approximated instead with METHOD = 'ncf'
%
%   V, W are bases for the constant basis space and parameter-varying
%       test space that define the parameter-varying oblique projection to
%       approximate balancing and truncation

% 02/12/2016 Julian Theis
%  changed default to deal with pss/pmat to avoid confusion when working on
%  data

if nargin ==2
    opt.method = 'default';
    opt.regTol = [];
    opt.regLam = [];
elseif ~isa(varargin{1},'struct')
    opt.method = varargin{1};
    opt.regTol = [];
    opt.regLam = [];
elseif isa(varargin{1},'struct')
    opt = varargin{1};
else
    error('invalid options')
end

if ~isa(sys,'pss')
    [ sysr, V, W, info ] = ssprojmr(sys, nr, varargin);
    return
end

[ny, nu] = size(sys);
ngrid = prod(size(sys.Domain));
 nx   = size(sys.a,1);
 
%% CALCULATE GRAMIANS at each grid point
% #########################################################################
fprintf(['Oblique Projection Model Order Reduction with ' opt.method ' Gramians\n'])

switch lower(opt.method)
    case 'default' % standard Lyapunov Equations with Hammarling Algorithm
        [Lc, Lo] = LOCAL_DefaultLyapunovEquations(sys);
                      
    case 'freq' % frequency weighted Lyapunov Equations with Hammarling Algorithm
        [Lc, Lo] = LOCAL_FrequencyWeightedLyapunovEquations(sys,opt.FreqWeightIn,opt.FreqWeightOut);
        
    otherwise
        error('No valid method selected. Valid methods are DEFAULT, FREQ')
end

% #########################################################################
%% Calculate Common Dominant Subspace USING CHOLESKY FACTORS
% #########################################################################
subspace_weighting = true;

[Uh, Sh, ~]   = svd(Lc'*Lo);
LU            = Lc*Uh(:,1:nr);

LUstack = zeros(nx,nr*ngrid);
for kk=1:ngrid
    if subspace_weighting
        LUstack(:,1+nr*(kk-1):nr*kk) = orth(LU.Data(:,:,kk))*Sh.Data(1:nr,1:nr,kk);
    else
        LUstack(:,1+nr*(kk-1):nr*kk) = orth(LU.Data(:,:,kk)); disp('orth(LU)')
    end
end
[Qbar, Sigbar, ~]   = svd(LUstack, 0);


% parameter-independent basis space
V = Qbar(:,1:nr);
% parameter-varying test space
  % as in ACC paper
    Y = Lo'*V;
    W = Lo/Y';      % W = Lo*Y/(Y'*Y);
    
% ######################################################################### 
%% Build Reduced-Order System
% ######################################################################### 

sysr = ss(W'*sys.a*V,...
          W'*sys.b,...
             sys.c*V,...
             sys.d);

info.Xo             = Lo*Lo';
info.Xc             = Lc*Lc';
info.subspaceweight = subspace_weighting;
sysr.OutputName     = sys.OutputName;
sysr.InputName      = sys.InputName;



% ######################################################################### 
%% AUX FUNCTIONS
% ######################################################################### 

% ######################################################################### 
%% LYAPUNOV SOLVER FUNCTIONS
% ######################################################################### 
function [Lci, Loi] = LOCAL_DefaultLyapunovEquations(sys)
% standard Lyapunov Equations with Hammarling Algorithm
[A,B,C,~] = ssdata(sys);

Lci  = lyapchol(A,B)';
err1 = norm(A*(Lci*Lci')+(Lci*Lci')*A'+B*B');
if double(lpvmax(err1))>1e-5; warning('Maximum Lyapunov (A,B) error %2.2e',double(lpvmax(err1))); end

Loi  = lyapchol(A',C')';
err2=norm(A'*(Loi*Loi')+(Loi*Loi')*A+C'*C);
if double(lpvmax(err2))>1e-5; warning('Lyapunov (A'',C'') error %2.2e',double(lpvmax(err2))); end


function [Lci, Loi] = LOCAL_FrequencyWeightedLyapunovEquations(sys,FreqWeightIn,FreqWeightOut)
[A,B,C,D] = ssdata(sys);
[ny, nu]  = size(sys);
 nx       = size(A,1);

if size(FreqWeightIn,1) ~= size(FreqWeightIn,1)
    error('frequency weight at input must be square')
elseif size(FreqWeightIn,1)==1
    FreqWeightIn = eye(nu)*ss(FreqWeightIn);
end
if size(FreqWeightOut,1) ~= size(FreqWeightOut,1)
    error('frequency weight at output must be square')
elseif size(FreqWeightOut,1)==1
    FreqWeightOut = eye(ny)*ss(FreqWeightOut);
end

% Frequency weighted Gramian Calculation Antoulas 2005
Sigma = ss( [A, B*FreqWeightIn.c; zeros(size(FreqWeightIn,'order'),nx) FreqWeightIn.a],...
    [B*FreqWeightIn.d; FreqWeightIn.b],...
    [C, D*FreqWeightIn.c],...
    D*FreqWeightIn.d);

LSigma = lyapchol(Sigma.a,Sigma.b)';
Lci = LSigma(1:nx,1:nx); 

% Frequency weighted Gramian Calculation Antoulas 2005
Sigma = ss( [A zeros(nx,size(FreqWeightOut,'order')) ; FreqWeightOut.b*C FreqWeightOut.a],...
    [B; FreqWeightOut.b*D],...
    [FreqWeightOut.d*C  FreqWeightOut.c],...
    FreqWeightOut.d*D);

LSigma = lyapchol(Sigma.a',Sigma.c')';
Loi = LSigma(1:nx,1:nx);
